from typing import Generator
from requests import Session
from time import sleep

USERNAME = ''
PASSWORD = ''
WORKSPACE = ''
REPO = ''
BASE_URL = f'https://api.bitbucket.org/internal/repositories/{WORKSPACE}/{REPO}/lfs'

def api_rate_limited(status_code: int) -> bool:
    if status_code == 429:
        print('Hit api rate limit, sleeping for 1 minute then attempting to resume...')
        sleep(60)
        return True
    return False

def authorized(status_code: int) -> bool:
    if status_code == 401:
        print('Received 401 response indicating misspelled/invalid credentials, '
              'please check your credentials and try again. Closing...')
        exit()
    elif status_code == 403:
        print('Received 403 response indicating missing permissions, '
              'please ensure you have granted yourself site admin and try again. '
              'Closing...')
        exit()
    return True

def oids() -> Generator[str, None, None]:
    with open('oids.txt', 'r') as in_file:
        for line in in_file:
            yield line.strip()

def remove_lfs_object(session: Session, oid: str) -> int:
    while True:
        r = session.delete(f'{BASE_URL}/{oid}')

        if not api_rate_limited(r.status_code) and authorized(r.status_code):
            return r.status_code

def handle_status_code(status_code: int, oid: str) -> None:
    if status_code == 204:
        # successful deletion
        with open('objects_removed.txt', '+a') as out:
            out.write(f'{oid}\n')
    elif status_code == 404:
        # object not found
        with open('objects_not_found.txt', '+a') as out:
            out.write(f'{oid}\n')

def main():
    session = Session()
    session.auth = (USERNAME, PASSWORD)

    objects_handled = 0
    print(f'Attempting to remove objects...')
    for oid in oids():
        status_code = remove_lfs_object(session, oid)
        handle_status_code(status_code, oid)
        objects_handled += 1
        if objects_handled % 50 == 0:
            print(f'\t{objects_handled} Objects Handled...')

if __name__ == '__main__':
    main()
